# indyeye_client.py for IndyEye 0.5.0
# Custom version-2023/02/23
import sys
import os

import requests
from PIL import Image
import json
import socket
from io import BytesIO
import pickle
import time

import grpc
from .proto.eyetask_pb2_grpc import EyeTaskStub
from .proto import eyetask_pb2 as EyeTask_pb2

ConnectUI = '/main/connect'
GetVersion = '/main/version'
GetApp = '/main/app'
RobotStatus = '/main/robot/status'
DisplayImage = '/main/robot/image'
ImageSize = '/main/robot/image/size'
RobotList = '/main/robot/info'
RobotConnect = '/main/robot/connect'
RobotDisconnect = '/main/robot/disconnect'
CameraIndex = '/main/camera/index'
GetCameraCount = '/main/camera/count'
CameraParameters = '/main/camera/param'
CameraAutoset = '/main/camera/autoset'
LogList = "/main/log/list"
PopMessage = "/message/popup"
Reboot = "/main/reboot"
License = "/main/license"
Locks = "/main/locks"

CamMode = '/calibration/camera/mode'
DetectMarker = '/calibration/camera/marker'
Viewpoint = '/calibration/viewpoint'
PoseNumber = '/calibration/viewpoint/count'
CalibationFileLIst = '/calibration/file/list/{type}'
CalibrationFile = '/calibration/file/{type}'
CalibrationDefault = '/calibration/default/{type}'
CalibrationStart = '/calibration/start'
CalibrationStop = '/calibration/stop'
CalibrationProgress = '/calibration/progress'
ShowWorkspace = '/calibration/camera/workspace'
DetectWorkspace = '/calibration/workspace/detect'
EditWorkspace = '/calibration/workspace/edit/{type}/{axis}'

GraphShow = '/detect/show'
GraphRun = '/detect/run'
GraphFileList = '/detect/file/list/graph'
GraphFile = '/detect/file/graph'
GraphDefault = '/detect/graph/default'
GraphEdit = '/detect/graph/{index}/{type}'
GraphFlow = '/detect/graph/flow'
GraphError = '/detect/graph/error'
GraphDictionary = '/detect/graph/wizard/dictionary/{lang}'
GraphWizard = '/detect/graph/wizard'
GraphTemplate = '/detect/graph/template'

ModuleFileList = '/detect/file/list/module/{index}'
ModuleFile = '/detect/file/module/{index}'
ModuleDictList = '/detect/module/dict/list'
ModuleHelpCheck = '/detect/module/help/check/{name}/{lang}'
ModuleHelpDocs = '/detect/module/help/docs/{name}/{lang}'
ModuleIcon = '/detect/module/icon/{name}'
ModuleAsset = '/detect/module/asset/{name}/{path:path}'
ModuleParam = '/detect/module/params/{index}/{type}'

GripperMark = '/detect/module/gripper/mark/{type}'
GripperDraw = '/detect/module/gripper/draw/{type}'
GripperList = '/detect/module/gripper/list'
GripperPickTest = '/detect/module/gripper/picktest'

CadFileList = '/cad/file/list'
CadFile = '/cad/file'

EYE_PORT_TCP = 2002  # default IndyEye Task Server Port
EYE_PORT_GRPC = 10511
custom_dat_len = 32  # default custom data length


class EyeCommand:
    DETECT = 0  # detect command
    RETRIEVE = 1  # no-detect, retrieve results one-by-one, with Select module ON RETRIEVE Option
    RESET = 2  # not used now
    GETLIST = 3  # get class list


class DetectKey:
    DETECTED = 'detected'
    PASSED = 'passed'
    CLASS = 'cls'
    REF_TO_END_TOOL = 'tar_ee_pose'       # 6D task position to pick object.
    REF_TO_PICK_POINT = 'tar_tool_pose'   # 6D Tool Center Position to pick object.
    REF_TO_OBJECT = 'tar_obj_pose'        # 6D Position of detected object.
    TOOL_INDEX = 'tool_idx'
    ERROR_STATE = 'error_state'
    ERROR_MODULE = 'error_module'
    CLASS_NAMES = 'class_names'

##
# @class IndyEyeClient
# @brief Rest API client
class IndyEye:
    def __init__(self, eye_ip):
        self.session = requests.session()
        self.eye_ip = eye_ip
        self.url = f"http://{eye_ip}:8088"
        self.version = self.get_version()
        if float(".".join(self.version.split(".")[:2])) >= 0.5:
            # initialize gRPC
            self.channel = grpc.insecure_channel('{}:{}'.format(eye_ip, EYE_PORT_GRPC),
                                                 options=[('grpc.max_receive_message_length', 100 * 1024 * 1024)])
            self.stub = EyeTaskStub(self.channel)
            self.request_id = 0
            self.run_command = self.run_command_grpc
        else:
            self.run_command = self.run_command_tcp

    ##
    # @brief indyeye communication function
    # @param cmd        command id, 0~3
    # @param cls        index of target class to detect. 0: ALL, 1~: index of class
    # @param pose_cmd   end-effector pose, [x,y,z,u,v,w] (unit: m, deg)
    # @param robot_ip   ip address of robot, for multi-robot use
    def run_command_grpc(self, cmd, cls=0, pose_cmd=None, robot_ip=None, **kwargs):
        self.request_id += 1
        if cmd == EyeCommand.GETLIST:
            resp = self.stub.GetClassList(EyeTask_pb2.Request(id=self.request_id))
            return {DetectKey.CLASS_NAMES: resp.class_names,
                    DetectKey.ERROR_STATE: resp.error_state}
        else:
            sdict = {'id': self.request_id, 'cls': int(cls)}
            if cmd == EyeCommand.DETECT:
                if pose_cmd is not None:
                    sdict['pose_cmd'] = pose_cmd
                if robot_ip is not None:
                    sdict['robot_ip'] = robot_ip
                resp = self.stub.Detect(EyeTask_pb2.DetectRequest(**sdict))
            elif cmd == EyeCommand.RETRIEVE:
                if robot_ip is not None:
                    sdict['robot_ip'] = robot_ip
                resp = self.stub.Retrieve(EyeTask_pb2.RetrieveRequest(**sdict))
            else:
                raise(NotImplementedError("Unknown command {}".format(cmd)))
            return {DetectKey.DETECTED: resp.detected,
                    DetectKey.PASSED: resp.passed,
                    DetectKey.CLASS: resp.cls,
                    DetectKey.REF_TO_END_TOOL: resp.tar_ee_pose,
                    DetectKey.REF_TO_PICK_POINT: resp.tar_tool_pose,
                    DetectKey.REF_TO_OBJECT: resp.tar_obj_pose,
                    DetectKey.TOOL_INDEX: resp.tool_idx,
                    DetectKey.ERROR_STATE: resp.error_state,
                    DetectKey.ERROR_MODULE: resp.error_module}

    ##
    # @brief indyeye communication function
    # @param cmd        command id, 0~3
    # @param cls        index of target class to detect. 0: ALL, 1~: index of class
    # @param pose_cmd   end-effector pose, [x,y,z,u,v,w] (unit: m, deg)
    # @param robot_ip   ip address of robot, for multi-robot use
    def run_command_tcp(self, cmd, cls, pose_cmd=None, robot_ip=None, **kwargs):
        sock = socket.socket(socket.AF_INET,
                             socket.SOCK_STREAM)  # SOCK_STREAM is TCP socket

        try:
            sock.connect((self.eye_ip, EYE_PORT_TCP))
            sdict = {'command': int(cmd), 'class_tar': int(cls), }
            if pose_cmd is not None:
                sdict['pose_cmd'] = pose_cmd
            if robot_ip is not None:
                sdict['robot_ip'] = robot_ip
            sdict.update(kwargs)
            sjson = json.dumps(sdict, cls=NumpyEncoder)
            sbuff = sjson.encode()
            sock.send(sbuff)
            #             print('sent: ',sjson)

            rbuff = sock.recv(1024)
            rjson = "".join(map(chr, rbuff))
            rdict = json.loads(rjson)
            if cmd == EyeCommand.GETLIST:
                rdict = {DetectKey.CLASS_NAMES: rdict['class_list'],
                         DetectKey.ERROR_STATE: rdict['STATE']}
            elif cmd in [EyeCommand.DETECT, EyeCommand.RETRIEVE]:
                rdict = {DetectKey.DETECTED: rdict['class_detect'] > 0,
                         DetectKey.PASSED: rdict['class_detect'] > 0 and rdict['STATE'] == 0,
                         DetectKey.CLASS: rdict['class_detect'],
                         DetectKey.REF_TO_END_TOOL: rdict['Tbe'],
                         DetectKey.REF_TO_PICK_POINT: rdict['Tbt'],
                         DetectKey.REF_TO_OBJECT: rdict['Tbo'],
                         DetectKey.TOOL_INDEX: rdict['tool_idx'],
                         DetectKey.ERROR_STATE: rdict['STATE']!=0,
                         DetectKey.ERROR_MODULE: None}
            else:
                raise(NotImplementedError("Unknown command {}".format(cmd)))
            return rdict
        except Exception as e:
            print("[ERROR] IndyEye Communication:")
            print(e)
        finally:
            sock.close()

    # '''
    # @param cls : object class (use get_object_list to get cls). 0: ALL, 1~: index of class
    # @param task_pos : current task position, [x,y,z,u,v,w] (unit: m, deg)
    # @return {
    #     #              'Tbe' : pick pose of end-effector in base coordinates
    #     #              'Tbt' : pick pose of tool point in base coordinates
    #     #              'Tbo' : detected object pose in base coordinates
    #     #             }
    def detect(self, cls, task_pos, mode=DetectKey.REF_TO_END_TOOL, **kwargs):
        rdict = self.run_command(cmd=EyeCommand.DETECT, cls=cls, pose_cmd=task_pos, **kwargs)
        if rdict is not None and rdict[DetectKey.DETECTED]:
            return rdict[mode]
        else:
            return None

    def detect_by_object_name(self, target_name, task_pos, mode=DetectKey.REF_TO_END_TOOL, **kwargs):
        '''
            target_name : object name (requires name of object from get_object_list),
            task_pos : current task position
        '''
        objs = self.get_object_dict()
        found_target = None
        found_target_idx = -1
        for key in objs.keys():
            if target_name == objs[key]:
                found_target_idx = key
        if found_target_idx != -1:
            result = self.run_command(cmd=EyeCommand.DETECT, cls=found_target_idx, pose_cmd=task_pos, **kwargs)
            if rdict is not None and rdict[DetectKey.DETECTED]:
                found_target = result[mode]
            else:
                found_target = None
        return found_target

    def retrieve(self, cls, task_pos=None, mode=DetectKey.REF_TO_END_TOOL, **kwargs):
        '''
            cls : object class ( use get_object_list to get cls),
            cls = 0 : all,

            task_pos : current task position
        '''
        rdict = self.run_command(cmd=EyeCommand.RETRIEVE, cls=cls, pose_cmd=task_pos, **kwargs)
        if rdict is not None and rdict[DetectKey.DETECTED]:
            return rdict[mode]
        else:
            return None

    def get_object_dict(self, **kwargs):
        rdict = self.run_command(cmd=EyeCommand.GETLIST, cls=0, pose_cmd=None, **kwargs)
        if rdict is not None and DetectKey.CLASS_NAMES in rdict:
            return {i_o+1: obj for i_o, obj in enumerate(rdict[DetectKey.CLASS_NAMES])}
        else:
            return None

    def get_version(self):
        return self.get(GetVersion)['data']

    ##
    # @return RGB image
    def image(self):
        resp = self.session.get(self.url + DisplayImage)
        binarycontent = BytesIO(resp.content)
        img = Image.open(binarycontent)
        return img

    ##
    # @return RGB image in local repository
    def repo_image(self, filename):
        resp = self.session.get(self.url + f'/download/image/{filename}.png')
        binarycontent = BytesIO(resp.content)
        img = Image.open(binarycontent)
        return img

    def get(self, command, timeout=3, **kwargs):
        resp = self.session.get(self.url + command, params=kwargs, timeout=timeout)
        return resp.json()

    def get_pkl(self, command, timeout=30, **kwargs):
        resp = self.session.get(self.url + command, params=kwargs, timeout=timeout)
        return pickle.load(pickle.io.BytesIO(resp.content))

    def put_pkl(self, command, timeout=30, **kwargs):
        resp = self.session.put(self.url + command, params=kwargs, timeout=timeout)
        return pickle.load(pickle.io.BytesIO(resp.content))

    def post(self, command, timeout=3, **kwargs):
        resp = self.session.post(self.url + command, params=kwargs, timeout=timeout)
        return resp.json()

    def put(self, command, timeout=3, **kwargs):
        resp = self.session.put(self.url + command, params=kwargs, timeout=timeout)
        return resp.json()

    def delete(self, command, timeout=3, **kwargs):
        resp = self.session.delete(self.url + command, params=kwargs, timeout=timeout)
        return resp.json()

    def add_module(self, index, type, name, verbose=True):
        check_error(self.post(GraphEdit.format(index=index, type=type), name=name),
                    f"Add Module {name}", verbose=verbose)
        graphflow = self.get(GraphFlow)['data']

        if str(index) not in graphflow["graph"]:
            return graphflow["starting_node"]
        else:
            ref_node = graphflow['graph'][str(index)]
            if type == "next":
                return ref_node["out"]
            elif type == "branch":
                return ref_node["branch"]
            elif type == "condition":
                return ref_node["condition"]
            else:
                raise (NotImplementedError(f"unknown connection type {type}"))

    def get_module_params(self, index, timeout=3):
        return self.get(ModuleParam.format(index=index, type='value'), timeout=timeout)['data']

    def get_module_param_range(self, index, param, timeout=1):
        return self.get(ModuleParam.format(index=index, type='available'), param=param, timeout=timeout)['data'][
            'available']

    def set_module_params(self, index, timeout=3, verbose=False, **kwargs):
        for param, value in kwargs.items():
            check_error(
                self.post(ModuleParam.format(index=index, type='value'), param=param, value=value, timeout=timeout),
                f"Param Set {param}={value}", verbose=verbose)

    def del_module(self, index, verbose=True):
        check_error(self.delete(GraphEdit.format(index=index, type="None")), "Delete Module", verbose=verbose)

    def clear_graph(self, verbose=True):
        check_error(self.delete(GraphEdit.format(index="all", type="None")), "Clear Graph", verbose=verbose)



##
# @brief check communication error and print [OK] message or raise error
def check_error(resp, message, verbose=True):
    if resp['error_code'] == 0:
        if verbose:
            print("[OK] " + message)
        return resp
    else:
        raise (RuntimeError(f"[ERROR {resp['error_code']}] " + message))




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: device.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import device_msgs_pb2 as device__msgs__pb2
import common_msgs_pb2 as common__msgs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='device.proto',
  package='Nrmk.IndyFramework',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0c\x64\x65vice.proto\x12\x12Nrmk.IndyFramework\x1a\x11\x64\x65vice_msgs.proto\x1a\x11\x63ommon_msgs.proto2\xd2\x0f\n\x06\x44\x65vice\x12J\n\tSetBrakes\x12\x1d.Nrmk.IndyFramework.MotorList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12H\n\x0bSetServoAll\x12\x19.Nrmk.IndyFramework.State\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12\x45\n\x08SetServo\x12\x19.Nrmk.IndyFramework.Servo\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12H\n\x05SetDI\x12\x1f.Nrmk.IndyFramework.DigitalList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12H\n\x05SetDO\x12\x1f.Nrmk.IndyFramework.DigitalList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12Q\n\x08SetEndDI\x12%.Nrmk.IndyFramework.EndtoolSignalList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12Q\n\x08SetEndDO\x12%.Nrmk.IndyFramework.EndtoolSignalList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12G\n\x05SetAI\x12\x1e.Nrmk.IndyFramework.AnalogList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12G\n\x05SetAO\x12\x1e.Nrmk.IndyFramework.AnalogList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12J\n\x08SetEndAI\x12\x1e.Nrmk.IndyFramework.AnalogList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12J\n\x08SetEndAO\x12\x1e.Nrmk.IndyFramework.AnalogList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12\x45\n\x05GetDI\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1f.Nrmk.IndyFramework.DigitalList\"\x00\x12\x45\n\x05GetDO\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1f.Nrmk.IndyFramework.DigitalList\"\x00\x12N\n\x08GetEndDI\x12\x19.Nrmk.IndyFramework.Empty\x1a%.Nrmk.IndyFramework.EndtoolSignalList\"\x00\x12N\n\x08GetEndDO\x12\x19.Nrmk.IndyFramework.Empty\x1a%.Nrmk.IndyFramework.EndtoolSignalList\"\x00\x12\x44\n\x05GetAI\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.AnalogList\"\x00\x12\x44\n\x05GetAO\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.AnalogList\"\x00\x12G\n\x08GetEndAI\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.AnalogList\"\x00\x12G\n\x08GetEndAO\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.AnalogList\"\x00\x12K\n\tGetEL5001\x12\x19.Nrmk.IndyFramework.Empty\x1a!.Nrmk.IndyFramework.GetEL5001Data\"\x00\x12K\n\tGetEL5101\x12\x19.Nrmk.IndyFramework.Empty\x1a!.Nrmk.IndyFramework.GetEL5101Data\"\x00\x12L\n\rGetDeviceInfo\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.DeviceInfo\"\x00\x12Z\n\x14GetBrakeControlStyle\x12\x19.Nrmk.IndyFramework.Empty\x1a%.Nrmk.IndyFramework.BrakeControlStyle\"\x00\x12S\n\x0fSetFTSensorType\x12 .Nrmk.IndyFramework.FTSensorType\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12P\n\x0fGetFTSensorType\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.FTSensorType\"\x00\x12P\n\x0fGetFTSensorData\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.FTSensorData\"\x00\x62\x06proto3'
  ,
  dependencies=[device__msgs__pb2.DESCRIPTOR,common__msgs__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_DEVICE = _descriptor.ServiceDescriptor(
  name='Device',
  full_name='Nrmk.IndyFramework.Device',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=75,
  serialized_end=2077,
  methods=[
  _descriptor.MethodDescriptor(
    name='SetBrakes',
    full_name='Nrmk.IndyFramework.Device.SetBrakes',
    index=0,
    containing_service=None,
    input_type=device__msgs__pb2._MOTORLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetServoAll',
    full_name='Nrmk.IndyFramework.Device.SetServoAll',
    index=1,
    containing_service=None,
    input_type=common__msgs__pb2._STATE,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetServo',
    full_name='Nrmk.IndyFramework.Device.SetServo',
    index=2,
    containing_service=None,
    input_type=device__msgs__pb2._SERVO,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDI',
    full_name='Nrmk.IndyFramework.Device.SetDI',
    index=3,
    containing_service=None,
    input_type=device__msgs__pb2._DIGITALLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDO',
    full_name='Nrmk.IndyFramework.Device.SetDO',
    index=4,
    containing_service=None,
    input_type=device__msgs__pb2._DIGITALLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetEndDI',
    full_name='Nrmk.IndyFramework.Device.SetEndDI',
    index=5,
    containing_service=None,
    input_type=device__msgs__pb2._ENDTOOLSIGNALLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetEndDO',
    full_name='Nrmk.IndyFramework.Device.SetEndDO',
    index=6,
    containing_service=None,
    input_type=device__msgs__pb2._ENDTOOLSIGNALLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetAI',
    full_name='Nrmk.IndyFramework.Device.SetAI',
    index=7,
    containing_service=None,
    input_type=device__msgs__pb2._ANALOGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetAO',
    full_name='Nrmk.IndyFramework.Device.SetAO',
    index=8,
    containing_service=None,
    input_type=device__msgs__pb2._ANALOGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetEndAI',
    full_name='Nrmk.IndyFramework.Device.SetEndAI',
    index=9,
    containing_service=None,
    input_type=device__msgs__pb2._ANALOGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetEndAO',
    full_name='Nrmk.IndyFramework.Device.SetEndAO',
    index=10,
    containing_service=None,
    input_type=device__msgs__pb2._ANALOGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDI',
    full_name='Nrmk.IndyFramework.Device.GetDI',
    index=11,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._DIGITALLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDO',
    full_name='Nrmk.IndyFramework.Device.GetDO',
    index=12,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._DIGITALLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEndDI',
    full_name='Nrmk.IndyFramework.Device.GetEndDI',
    index=13,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ENDTOOLSIGNALLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEndDO',
    full_name='Nrmk.IndyFramework.Device.GetEndDO',
    index=14,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ENDTOOLSIGNALLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetAI',
    full_name='Nrmk.IndyFramework.Device.GetAI',
    index=15,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ANALOGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetAO',
    full_name='Nrmk.IndyFramework.Device.GetAO',
    index=16,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ANALOGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEndAI',
    full_name='Nrmk.IndyFramework.Device.GetEndAI',
    index=17,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ANALOGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEndAO',
    full_name='Nrmk.IndyFramework.Device.GetEndAO',
    index=18,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._ANALOGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEL5001',
    full_name='Nrmk.IndyFramework.Device.GetEL5001',
    index=19,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._GETEL5001DATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetEL5101',
    full_name='Nrmk.IndyFramework.Device.GetEL5101',
    index=20,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._GETEL5101DATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDeviceInfo',
    full_name='Nrmk.IndyFramework.Device.GetDeviceInfo',
    index=21,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._DEVICEINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBrakeControlStyle',
    full_name='Nrmk.IndyFramework.Device.GetBrakeControlStyle',
    index=22,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._BRAKECONTROLSTYLE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetFTSensorType',
    full_name='Nrmk.IndyFramework.Device.SetFTSensorType',
    index=23,
    containing_service=None,
    input_type=device__msgs__pb2._FTSENSORTYPE,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFTSensorType',
    full_name='Nrmk.IndyFramework.Device.GetFTSensorType',
    index=24,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._FTSENSORTYPE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFTSensorData',
    full_name='Nrmk.IndyFramework.Device.GetFTSensorData',
    index=25,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=device__msgs__pb2._FTSENSORDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DEVICE)

DESCRIPTOR.services_by_name['Device'] = _DEVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: moby.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import moby_msgs_pb2 as moby__msgs__pb2
import common_msgs_pb2 as common__msgs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='moby.proto',
  package='Nrmk.IndyFramework',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\nmoby.proto\x12\x12Nrmk.IndyFramework\x1a\x0fmoby_msgs.proto\x1a\x11\x63ommon_msgs.proto2\x82\x16\n\x04Moby\x12J\n\x0cGetMobyState\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1d.Nrmk.IndyFramework.MobyState\"\x00\x12T\n\x11GetMobyErrorState\x12\x19.Nrmk.IndyFramework.Empty\x1a\".Nrmk.IndyFramework.MobyErrorState\"\x00\x12\x41\n\x07Recover\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12H\n\x0bGetMobyPose\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1c.Nrmk.IndyFramework.MobyPose\"\x00\x12\x46\n\nGetMobyVel\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1b.Nrmk.IndyFramework.MobyVel\"\x00\x12G\n\rResetMobyPose\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12U\n\x13GetRotationAngleDeg\x12\x19.Nrmk.IndyFramework.Empty\x1a!.Nrmk.IndyFramework.SwerveDoubles\"\x00\x12O\n\rGetDriveSpeed\x12\x19.Nrmk.IndyFramework.Empty\x1a!.Nrmk.IndyFramework.SwerveDoubles\"\x00\x12J\n\x0cGetTargetVel\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1d.Nrmk.IndyFramework.TargetVel\"\x00\x12R\n\x14GetRotationZeroCount\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1d.Nrmk.IndyFramework.ZeroCount\"\x00\x12J\n\x0bGetGyroData\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1e.Nrmk.IndyFramework.DoubleVals\"\x00\x12I\n\x0fResetGyroSensor\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12J\n\x0eUseGyroForOdom\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12K\n\x0fGetGyroFullData\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1b.Nrmk.IndyFramework.IMUData\"\x00\x12J\n\x0fGetIRSensorData\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1a.Nrmk.IndyFramework.IRData\"\x00\x12J\n\x0fGetUSSensorData\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1a.Nrmk.IndyFramework.USData\"\x00\x12\x46\n\nGetBMSData\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1b.Nrmk.IndyFramework.BMSData\"\x00\x12L\n\x0eSetStepControl\x12\x1d.Nrmk.IndyFramework.TargetVel\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12\x44\n\nStopMotion\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12U\n\x13SetRotationAngleDeg\x12!.Nrmk.IndyFramework.SwerveDoubles\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12L\n\nDriveWheel\x12!.Nrmk.IndyFramework.SwerveDoubles\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12P\n\x16SetZeroPosAsCurrentPos\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12P\n\x11SetRotationVelAcc\x12\x1e.Nrmk.IndyFramework.DoubleVals\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12R\n\x17SetRotationInterpolator\x12\x1a.Nrmk.IndyFramework.IntVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12M\n\x0eSetDriveAccDec\x12\x1e.Nrmk.IndyFramework.DoubleVals\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12U\n\x19SetDriveInterpolatorOnOff\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12T\n\x19SetRotationControllerType\x12\x1a.Nrmk.IndyFramework.IntVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12N\n\x12SetForceKinematics\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12N\n\x12GetForceKinematics\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1b.Nrmk.IndyFramework.BoolVal\"\x00\x12G\n\x0bPauseBumper\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12J\n\x0eTurnLightOnOff\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12I\n\rTurnBuzzOnOff\x12\x1b.Nrmk.IndyFramework.BoolVal\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12P\n\x0fSetControlParam\x12 .Nrmk.IndyFramework.RotationGain\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12Q\n\x0fGetControlParam\x12\x1a.Nrmk.IndyFramework.IntVal\x1a .Nrmk.IndyFramework.RotationGain\"\x00\x12H\n\x0eStartRTLogging\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x12\x46\n\x0c\x45ndRTLogging\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Empty\"\x00\x62\x06proto3'
  ,
  dependencies=[moby__msgs__pb2.DESCRIPTOR,common__msgs__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_MOBY = _descriptor.ServiceDescriptor(
  name='Moby',
  full_name='Nrmk.IndyFramework.Moby',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=71,
  serialized_end=2889,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetMobyState',
    full_name='Nrmk.IndyFramework.Moby.GetMobyState',
    index=0,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._MOBYSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMobyErrorState',
    full_name='Nrmk.IndyFramework.Moby.GetMobyErrorState',
    index=1,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._MOBYERRORSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Recover',
    full_name='Nrmk.IndyFramework.Moby.Recover',
    index=2,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMobyPose',
    full_name='Nrmk.IndyFramework.Moby.GetMobyPose',
    index=3,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._MOBYPOSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMobyVel',
    full_name='Nrmk.IndyFramework.Moby.GetMobyVel',
    index=4,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._MOBYVEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ResetMobyPose',
    full_name='Nrmk.IndyFramework.Moby.ResetMobyPose',
    index=5,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetRotationAngleDeg',
    full_name='Nrmk.IndyFramework.Moby.GetRotationAngleDeg',
    index=6,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._SWERVEDOUBLES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDriveSpeed',
    full_name='Nrmk.IndyFramework.Moby.GetDriveSpeed',
    index=7,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._SWERVEDOUBLES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetTargetVel',
    full_name='Nrmk.IndyFramework.Moby.GetTargetVel',
    index=8,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._TARGETVEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetRotationZeroCount',
    full_name='Nrmk.IndyFramework.Moby.GetRotationZeroCount',
    index=9,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._ZEROCOUNT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetGyroData',
    full_name='Nrmk.IndyFramework.Moby.GetGyroData',
    index=10,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._DOUBLEVALS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ResetGyroSensor',
    full_name='Nrmk.IndyFramework.Moby.ResetGyroSensor',
    index=11,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UseGyroForOdom',
    full_name='Nrmk.IndyFramework.Moby.UseGyroForOdom',
    index=12,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetGyroFullData',
    full_name='Nrmk.IndyFramework.Moby.GetGyroFullData',
    index=13,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._IMUDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetIRSensorData',
    full_name='Nrmk.IndyFramework.Moby.GetIRSensorData',
    index=14,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._IRDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUSSensorData',
    full_name='Nrmk.IndyFramework.Moby.GetUSSensorData',
    index=15,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._USDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBMSData',
    full_name='Nrmk.IndyFramework.Moby.GetBMSData',
    index=16,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._BMSDATA,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetStepControl',
    full_name='Nrmk.IndyFramework.Moby.SetStepControl',
    index=17,
    containing_service=None,
    input_type=moby__msgs__pb2._TARGETVEL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StopMotion',
    full_name='Nrmk.IndyFramework.Moby.StopMotion',
    index=18,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRotationAngleDeg',
    full_name='Nrmk.IndyFramework.Moby.SetRotationAngleDeg',
    index=19,
    containing_service=None,
    input_type=moby__msgs__pb2._SWERVEDOUBLES,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DriveWheel',
    full_name='Nrmk.IndyFramework.Moby.DriveWheel',
    index=20,
    containing_service=None,
    input_type=moby__msgs__pb2._SWERVEDOUBLES,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetZeroPosAsCurrentPos',
    full_name='Nrmk.IndyFramework.Moby.SetZeroPosAsCurrentPos',
    index=21,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRotationVelAcc',
    full_name='Nrmk.IndyFramework.Moby.SetRotationVelAcc',
    index=22,
    containing_service=None,
    input_type=moby__msgs__pb2._DOUBLEVALS,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRotationInterpolator',
    full_name='Nrmk.IndyFramework.Moby.SetRotationInterpolator',
    index=23,
    containing_service=None,
    input_type=moby__msgs__pb2._INTVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDriveAccDec',
    full_name='Nrmk.IndyFramework.Moby.SetDriveAccDec',
    index=24,
    containing_service=None,
    input_type=moby__msgs__pb2._DOUBLEVALS,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDriveInterpolatorOnOff',
    full_name='Nrmk.IndyFramework.Moby.SetDriveInterpolatorOnOff',
    index=25,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRotationControllerType',
    full_name='Nrmk.IndyFramework.Moby.SetRotationControllerType',
    index=26,
    containing_service=None,
    input_type=moby__msgs__pb2._INTVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetForceKinematics',
    full_name='Nrmk.IndyFramework.Moby.SetForceKinematics',
    index=27,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetForceKinematics',
    full_name='Nrmk.IndyFramework.Moby.GetForceKinematics',
    index=28,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=moby__msgs__pb2._BOOLVAL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PauseBumper',
    full_name='Nrmk.IndyFramework.Moby.PauseBumper',
    index=29,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TurnLightOnOff',
    full_name='Nrmk.IndyFramework.Moby.TurnLightOnOff',
    index=30,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TurnBuzzOnOff',
    full_name='Nrmk.IndyFramework.Moby.TurnBuzzOnOff',
    index=31,
    containing_service=None,
    input_type=moby__msgs__pb2._BOOLVAL,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetControlParam',
    full_name='Nrmk.IndyFramework.Moby.SetControlParam',
    index=32,
    containing_service=None,
    input_type=moby__msgs__pb2._ROTATIONGAIN,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetControlParam',
    full_name='Nrmk.IndyFramework.Moby.GetControlParam',
    index=33,
    containing_service=None,
    input_type=moby__msgs__pb2._INTVAL,
    output_type=moby__msgs__pb2._ROTATIONGAIN,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StartRTLogging',
    full_name='Nrmk.IndyFramework.Moby.StartRTLogging',
    index=34,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='EndRTLogging',
    full_name='Nrmk.IndyFramework.Moby.EndRTLogging',
    index=35,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=common__msgs__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MOBY)

DESCRIPTOR.services_by_name['Moby'] = _MOBY

# @@protoc_insertion_point(module_scope)
